# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ActorTableConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'offset': 'int',
        'page_size': 'int',
        'actor_id': 'str',
        'name': 'str',
        'states': 'list[ActorState]',
        'job_id': 'str',
        'text': 'str'
    }

    attribute_map = {
        'offset': 'offset',
        'page_size': 'page_size',
        'actor_id': 'actor_id',
        'name': 'name',
        'states': 'states',
        'job_id': 'job_id',
        'text': 'text'
    }

    def __init__(self, offset=None, page_size=None, actor_id=None, name=None, states=[], job_id=None, text=None, local_vars_configuration=None):  # noqa: E501
        """ActorTableConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._offset = None
        self._page_size = None
        self._actor_id = None
        self._name = None
        self._states = None
        self._job_id = None
        self._text = None
        self.discriminator = None

        if offset is not None:
            self.offset = offset
        if page_size is not None:
            self.page_size = page_size
        if actor_id is not None:
            self.actor_id = actor_id
        if name is not None:
            self.name = name
        if states is not None:
            self.states = states
        if job_id is not None:
            self.job_id = job_id
        if text is not None:
            self.text = text

    @property
    def offset(self):
        """Gets the offset of this ActorTableConfig.  # noqa: E501


        :return: The offset of this ActorTableConfig.  # noqa: E501
        :rtype: int
        """
        return self._offset

    @offset.setter
    def offset(self, offset):
        """Sets the offset of this ActorTableConfig.


        :param offset: The offset of this ActorTableConfig.  # noqa: E501
        :type: int
        """

        self._offset = offset

    @property
    def page_size(self):
        """Gets the page_size of this ActorTableConfig.  # noqa: E501


        :return: The page_size of this ActorTableConfig.  # noqa: E501
        :rtype: int
        """
        return self._page_size

    @page_size.setter
    def page_size(self, page_size):
        """Sets the page_size of this ActorTableConfig.


        :param page_size: The page_size of this ActorTableConfig.  # noqa: E501
        :type: int
        """

        self._page_size = page_size

    @property
    def actor_id(self):
        """Gets the actor_id of this ActorTableConfig.  # noqa: E501


        :return: The actor_id of this ActorTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._actor_id

    @actor_id.setter
    def actor_id(self, actor_id):
        """Sets the actor_id of this ActorTableConfig.


        :param actor_id: The actor_id of this ActorTableConfig.  # noqa: E501
        :type: str
        """

        self._actor_id = actor_id

    @property
    def name(self):
        """Gets the name of this ActorTableConfig.  # noqa: E501


        :return: The name of this ActorTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ActorTableConfig.


        :param name: The name of this ActorTableConfig.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def states(self):
        """Gets the states of this ActorTableConfig.  # noqa: E501


        :return: The states of this ActorTableConfig.  # noqa: E501
        :rtype: list[ActorState]
        """
        return self._states

    @states.setter
    def states(self, states):
        """Sets the states of this ActorTableConfig.


        :param states: The states of this ActorTableConfig.  # noqa: E501
        :type: list[ActorState]
        """

        self._states = states

    @property
    def job_id(self):
        """Gets the job_id of this ActorTableConfig.  # noqa: E501


        :return: The job_id of this ActorTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this ActorTableConfig.


        :param job_id: The job_id of this ActorTableConfig.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def text(self):
        """Gets the text of this ActorTableConfig.  # noqa: E501


        :return: The text of this ActorTableConfig.  # noqa: E501
        :rtype: str
        """
        return self._text

    @text.setter
    def text(self, text):
        """Sets the text of this ActorTableConfig.


        :param text: The text of this ActorTableConfig.  # noqa: E501
        :type: str
        """

        self._text = text

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ActorTableConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ActorTableConfig):
            return True

        return self.to_dict() != other.to_dict()
