# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class TracingConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'exporter_import_path': 'str',
        'sampling_ratio': 'float',
        'enabled': 'bool'
    }

    attribute_map = {
        'exporter_import_path': 'exporter_import_path',
        'sampling_ratio': 'sampling_ratio',
        'enabled': 'enabled'
    }

    def __init__(self, exporter_import_path=None, sampling_ratio=1.0, enabled=True, local_vars_configuration=None):  # noqa: E501
        """TracingConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._exporter_import_path = None
        self._sampling_ratio = None
        self._enabled = None
        self.discriminator = None

        if exporter_import_path is not None:
            self.exporter_import_path = exporter_import_path
        if sampling_ratio is not None:
            self.sampling_ratio = sampling_ratio
        if enabled is not None:
            self.enabled = enabled

    @property
    def exporter_import_path(self):
        """Gets the exporter_import_path of this TracingConfig.  # noqa: E501

        Path to exporter function. Should be of the form \"module.submodule_1...submodule_n.export_tracing\". This is equivalent to \"from module.submodule_1...submodule_n import export_tracing. Exporter function takes no arguments and returns List[SpanProcessor].  # noqa: E501

        :return: The exporter_import_path of this TracingConfig.  # noqa: E501
        :rtype: str
        """
        return self._exporter_import_path

    @exporter_import_path.setter
    def exporter_import_path(self, exporter_import_path):
        """Sets the exporter_import_path of this TracingConfig.

        Path to exporter function. Should be of the form \"module.submodule_1...submodule_n.export_tracing\". This is equivalent to \"from module.submodule_1...submodule_n import export_tracing. Exporter function takes no arguments and returns List[SpanProcessor].  # noqa: E501

        :param exporter_import_path: The exporter_import_path of this TracingConfig.  # noqa: E501
        :type: str
        """

        self._exporter_import_path = exporter_import_path

    @property
    def sampling_ratio(self):
        """Gets the sampling_ratio of this TracingConfig.  # noqa: E501

        Ratio of traces to record and export. Reducing the ratio can help with minimizing serving latency and storage overhead.  # noqa: E501

        :return: The sampling_ratio of this TracingConfig.  # noqa: E501
        :rtype: float
        """
        return self._sampling_ratio

    @sampling_ratio.setter
    def sampling_ratio(self, sampling_ratio):
        """Sets the sampling_ratio of this TracingConfig.

        Ratio of traces to record and export. Reducing the ratio can help with minimizing serving latency and storage overhead.  # noqa: E501

        :param sampling_ratio: The sampling_ratio of this TracingConfig.  # noqa: E501
        :type: float
        """

        self._sampling_ratio = sampling_ratio

    @property
    def enabled(self):
        """Gets the enabled of this TracingConfig.  # noqa: E501

        Flag to enable tracing in Serve application. If tracing is enabled and no `exporter_import_path` path is defined, then the default tracing exporter will be used to export traces to serve logs.  # noqa: E501

        :return: The enabled of this TracingConfig.  # noqa: E501
        :rtype: bool
        """
        return self._enabled

    @enabled.setter
    def enabled(self, enabled):
        """Sets the enabled of this TracingConfig.

        Flag to enable tracing in Serve application. If tracing is enabled and no `exporter_import_path` path is defined, then the default tracing exporter will be used to export traces to serve logs.  # noqa: E501

        :param enabled: The enabled of this TracingConfig.  # noqa: E501
        :type: bool
        """

        self._enabled = enabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, TracingConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, TracingConfig):
            return True

        return self.to_dict() != other.to_dict()
