# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ActorGroupedAggregateMetrics(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'total': 'int',
        'count_dependencies_unready': 'int',
        'count_pending_creation': 'int',
        'count_alive': 'int',
        'count_restarting': 'int',
        'count_dead': 'int'
    }

    attribute_map = {
        'total': 'total',
        'count_dependencies_unready': 'count_dependencies_unready',
        'count_pending_creation': 'count_pending_creation',
        'count_alive': 'count_alive',
        'count_restarting': 'count_restarting',
        'count_dead': 'count_dead'
    }

    def __init__(self, total=None, count_dependencies_unready=None, count_pending_creation=None, count_alive=None, count_restarting=None, count_dead=None, local_vars_configuration=None):  # noqa: E501
        """ActorGroupedAggregateMetrics - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._total = None
        self._count_dependencies_unready = None
        self._count_pending_creation = None
        self._count_alive = None
        self._count_restarting = None
        self._count_dead = None
        self.discriminator = None

        self.total = total
        self.count_dependencies_unready = count_dependencies_unready
        self.count_pending_creation = count_pending_creation
        self.count_alive = count_alive
        self.count_restarting = count_restarting
        self.count_dead = count_dead

    @property
    def total(self):
        """Gets the total of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The total of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this ActorGroupedAggregateMetrics.


        :param total: The total of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and total is None:  # noqa: E501
            raise ValueError("Invalid value for `total`, must not be `None`")  # noqa: E501

        self._total = total

    @property
    def count_dependencies_unready(self):
        """Gets the count_dependencies_unready of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The count_dependencies_unready of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._count_dependencies_unready

    @count_dependencies_unready.setter
    def count_dependencies_unready(self, count_dependencies_unready):
        """Sets the count_dependencies_unready of this ActorGroupedAggregateMetrics.


        :param count_dependencies_unready: The count_dependencies_unready of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and count_dependencies_unready is None:  # noqa: E501
            raise ValueError("Invalid value for `count_dependencies_unready`, must not be `None`")  # noqa: E501

        self._count_dependencies_unready = count_dependencies_unready

    @property
    def count_pending_creation(self):
        """Gets the count_pending_creation of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The count_pending_creation of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._count_pending_creation

    @count_pending_creation.setter
    def count_pending_creation(self, count_pending_creation):
        """Sets the count_pending_creation of this ActorGroupedAggregateMetrics.


        :param count_pending_creation: The count_pending_creation of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and count_pending_creation is None:  # noqa: E501
            raise ValueError("Invalid value for `count_pending_creation`, must not be `None`")  # noqa: E501

        self._count_pending_creation = count_pending_creation

    @property
    def count_alive(self):
        """Gets the count_alive of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The count_alive of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._count_alive

    @count_alive.setter
    def count_alive(self, count_alive):
        """Sets the count_alive of this ActorGroupedAggregateMetrics.


        :param count_alive: The count_alive of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and count_alive is None:  # noqa: E501
            raise ValueError("Invalid value for `count_alive`, must not be `None`")  # noqa: E501

        self._count_alive = count_alive

    @property
    def count_restarting(self):
        """Gets the count_restarting of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The count_restarting of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._count_restarting

    @count_restarting.setter
    def count_restarting(self, count_restarting):
        """Sets the count_restarting of this ActorGroupedAggregateMetrics.


        :param count_restarting: The count_restarting of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and count_restarting is None:  # noqa: E501
            raise ValueError("Invalid value for `count_restarting`, must not be `None`")  # noqa: E501

        self._count_restarting = count_restarting

    @property
    def count_dead(self):
        """Gets the count_dead of this ActorGroupedAggregateMetrics.  # noqa: E501


        :return: The count_dead of this ActorGroupedAggregateMetrics.  # noqa: E501
        :rtype: int
        """
        return self._count_dead

    @count_dead.setter
    def count_dead(self, count_dead):
        """Sets the count_dead of this ActorGroupedAggregateMetrics.


        :param count_dead: The count_dead of this ActorGroupedAggregateMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and count_dead is None:  # noqa: E501
            raise ValueError("Invalid value for `count_dead`, must not be `None`")  # noqa: E501

        self._count_dead = count_dead

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ActorGroupedAggregateMetrics):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ActorGroupedAggregateMetrics):
            return True

        return self.to_dict() != other.to_dict()
