# AUTOGENERATED - modify shared_anyscale_util in root directory to make changes
import json
import os

import yaml


# TODO(nikita): Remove CLUSTER_YAML_TEMPLATE and CLUSTER_CONFIG_TEMPLATE_STR once we remove cluster config
# from projects model.
with open(os.path.join(os.path.dirname(__file__), "default_anyscale_aws.yaml")) as f:
    CLUSTER_YAML_TEMPLATE = f.read()

CLUSTER_CONFIG_TEMPLATE_STR = json.dumps(
    yaml.load(CLUSTER_YAML_TEMPLATE, Loader=yaml.SafeLoader)
)
