# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ConnectionMetadata(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'owner': 'str',
        'full_name': 'str',
        'workspace_url': 'str',
        'client_id': 'str'
    }

    attribute_map = {
        'owner': 'owner',
        'full_name': 'full_name',
        'workspace_url': 'workspace_url',
        'client_id': 'client_id'
    }

    def __init__(self, owner=None, full_name=None, workspace_url=None, client_id=None, local_vars_configuration=None):  # noqa: E501
        """ConnectionMetadata - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._owner = None
        self._full_name = None
        self._workspace_url = None
        self._client_id = None
        self.discriminator = None

        if owner is not None:
            self.owner = owner
        if full_name is not None:
            self.full_name = full_name
        if workspace_url is not None:
            self.workspace_url = workspace_url
        if client_id is not None:
            self.client_id = client_id

    @property
    def owner(self):
        """Gets the owner of this ConnectionMetadata.  # noqa: E501

        Owner of the entity  # noqa: E501

        :return: The owner of this ConnectionMetadata.  # noqa: E501
        :rtype: str
        """
        return self._owner

    @owner.setter
    def owner(self, owner):
        """Sets the owner of this ConnectionMetadata.

        Owner of the entity  # noqa: E501

        :param owner: The owner of this ConnectionMetadata.  # noqa: E501
        :type: str
        """

        self._owner = owner

    @property
    def full_name(self):
        """Gets the full_name of this ConnectionMetadata.  # noqa: E501

        Fully qualified name of the entity  # noqa: E501

        :return: The full_name of this ConnectionMetadata.  # noqa: E501
        :rtype: str
        """
        return self._full_name

    @full_name.setter
    def full_name(self, full_name):
        """Sets the full_name of this ConnectionMetadata.

        Fully qualified name of the entity  # noqa: E501

        :param full_name: The full_name of this ConnectionMetadata.  # noqa: E501
        :type: str
        """

        self._full_name = full_name

    @property
    def workspace_url(self):
        """Gets the workspace_url of this ConnectionMetadata.  # noqa: E501

        Workspace URL for the connection  # noqa: E501

        :return: The workspace_url of this ConnectionMetadata.  # noqa: E501
        :rtype: str
        """
        return self._workspace_url

    @workspace_url.setter
    def workspace_url(self, workspace_url):
        """Sets the workspace_url of this ConnectionMetadata.

        Workspace URL for the connection  # noqa: E501

        :param workspace_url: The workspace_url of this ConnectionMetadata.  # noqa: E501
        :type: str
        """

        self._workspace_url = workspace_url

    @property
    def client_id(self):
        """Gets the client_id of this ConnectionMetadata.  # noqa: E501

        OAuth client ID  # noqa: E501

        :return: The client_id of this ConnectionMetadata.  # noqa: E501
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """Sets the client_id of this ConnectionMetadata.

        OAuth client ID  # noqa: E501

        :param client_id: The client_id of this ConnectionMetadata.  # noqa: E501
        :type: str
        """

        self._client_id = client_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConnectionMetadata):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConnectionMetadata):
            return True

        return self.to_dict() != other.to_dict()
