# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationConfigurationResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'customer_billing_type': 'str',
        'invite_code': 'str',
        'registration_url': 'str',
        'expires_at': 'datetime',
        'description': 'str',
        'contract_end_date': 'datetime'
    }

    attribute_map = {
        'customer_billing_type': 'customer_billing_type',
        'invite_code': 'invite_code',
        'registration_url': 'registration_url',
        'expires_at': 'expires_at',
        'description': 'description',
        'contract_end_date': 'contract_end_date'
    }

    def __init__(self, customer_billing_type=None, invite_code=None, registration_url=None, expires_at=None, description=None, contract_end_date=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationConfigurationResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._customer_billing_type = None
        self._invite_code = None
        self._registration_url = None
        self._expires_at = None
        self._description = None
        self._contract_end_date = None
        self.discriminator = None

        self.customer_billing_type = customer_billing_type
        self.invite_code = invite_code
        self.registration_url = registration_url
        if expires_at is not None:
            self.expires_at = expires_at
        if description is not None:
            self.description = description
        if contract_end_date is not None:
            self.contract_end_date = contract_end_date

    @property
    def customer_billing_type(self):
        """Gets the customer_billing_type of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The customer_billing_type of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._customer_billing_type

    @customer_billing_type.setter
    def customer_billing_type(self, customer_billing_type):
        """Sets the customer_billing_type of this OrganizationConfigurationResponse.


        :param customer_billing_type: The customer_billing_type of this OrganizationConfigurationResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and customer_billing_type is None:  # noqa: E501
            raise ValueError("Invalid value for `customer_billing_type`, must not be `None`")  # noqa: E501

        self._customer_billing_type = customer_billing_type

    @property
    def invite_code(self):
        """Gets the invite_code of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The invite_code of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._invite_code

    @invite_code.setter
    def invite_code(self, invite_code):
        """Sets the invite_code of this OrganizationConfigurationResponse.


        :param invite_code: The invite_code of this OrganizationConfigurationResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and invite_code is None:  # noqa: E501
            raise ValueError("Invalid value for `invite_code`, must not be `None`")  # noqa: E501

        self._invite_code = invite_code

    @property
    def registration_url(self):
        """Gets the registration_url of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The registration_url of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._registration_url

    @registration_url.setter
    def registration_url(self, registration_url):
        """Sets the registration_url of this OrganizationConfigurationResponse.


        :param registration_url: The registration_url of this OrganizationConfigurationResponse.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and registration_url is None:  # noqa: E501
            raise ValueError("Invalid value for `registration_url`, must not be `None`")  # noqa: E501

        self._registration_url = registration_url

    @property
    def expires_at(self):
        """Gets the expires_at of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The expires_at of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: datetime
        """
        return self._expires_at

    @expires_at.setter
    def expires_at(self, expires_at):
        """Sets the expires_at of this OrganizationConfigurationResponse.


        :param expires_at: The expires_at of this OrganizationConfigurationResponse.  # noqa: E501
        :type: datetime
        """

        self._expires_at = expires_at

    @property
    def description(self):
        """Gets the description of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The description of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this OrganizationConfigurationResponse.


        :param description: The description of this OrganizationConfigurationResponse.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def contract_end_date(self):
        """Gets the contract_end_date of this OrganizationConfigurationResponse.  # noqa: E501


        :return: The contract_end_date of this OrganizationConfigurationResponse.  # noqa: E501
        :rtype: datetime
        """
        return self._contract_end_date

    @contract_end_date.setter
    def contract_end_date(self, contract_end_date):
        """Sets the contract_end_date of this OrganizationConfigurationResponse.


        :param contract_end_date: The contract_end_date of this OrganizationConfigurationResponse.  # noqa: E501
        :type: datetime
        """

        self._contract_end_date = contract_end_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationConfigurationResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationConfigurationResponse):
            return True

        return self.to_dict() != other.to_dict()
