# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OperatorEvent(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'dataset_id': 'str',
        'event_time': 'float',
        'event_type': 'OperatorEventType',
        'message': 'str'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'dataset_id': 'dataset_id',
        'event_time': 'event_time',
        'event_type': 'event_type',
        'message': 'message'
    }

    def __init__(self, id=None, name=None, dataset_id=None, event_time=None, event_type=None, message=None, local_vars_configuration=None):  # noqa: E501
        """OperatorEvent - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._dataset_id = None
        self._event_time = None
        self._event_type = None
        self._message = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.dataset_id = dataset_id
        self.event_time = event_time
        self.event_type = event_type
        self.message = message

    @property
    def id(self):
        """Gets the id of this OperatorEvent.  # noqa: E501


        :return: The id of this OperatorEvent.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this OperatorEvent.


        :param id: The id of this OperatorEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this OperatorEvent.  # noqa: E501


        :return: The name of this OperatorEvent.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this OperatorEvent.


        :param name: The name of this OperatorEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def dataset_id(self):
        """Gets the dataset_id of this OperatorEvent.  # noqa: E501


        :return: The dataset_id of this OperatorEvent.  # noqa: E501
        :rtype: str
        """
        return self._dataset_id

    @dataset_id.setter
    def dataset_id(self, dataset_id):
        """Sets the dataset_id of this OperatorEvent.


        :param dataset_id: The dataset_id of this OperatorEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and dataset_id is None:  # noqa: E501
            raise ValueError("Invalid value for `dataset_id`, must not be `None`")  # noqa: E501

        self._dataset_id = dataset_id

    @property
    def event_time(self):
        """Gets the event_time of this OperatorEvent.  # noqa: E501


        :return: The event_time of this OperatorEvent.  # noqa: E501
        :rtype: float
        """
        return self._event_time

    @event_time.setter
    def event_time(self, event_time):
        """Sets the event_time of this OperatorEvent.


        :param event_time: The event_time of this OperatorEvent.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and event_time is None:  # noqa: E501
            raise ValueError("Invalid value for `event_time`, must not be `None`")  # noqa: E501

        self._event_time = event_time

    @property
    def event_type(self):
        """Gets the event_type of this OperatorEvent.  # noqa: E501


        :return: The event_type of this OperatorEvent.  # noqa: E501
        :rtype: OperatorEventType
        """
        return self._event_type

    @event_type.setter
    def event_type(self, event_type):
        """Sets the event_type of this OperatorEvent.


        :param event_type: The event_type of this OperatorEvent.  # noqa: E501
        :type: OperatorEventType
        """
        if self.local_vars_configuration.client_side_validation and event_type is None:  # noqa: E501
            raise ValueError("Invalid value for `event_type`, must not be `None`")  # noqa: E501

        self._event_type = event_type

    @property
    def message(self):
        """Gets the message of this OperatorEvent.  # noqa: E501


        :return: The message of this OperatorEvent.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this OperatorEvent.


        :param message: The message of this OperatorEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and message is None:  # noqa: E501
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OperatorEvent):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OperatorEvent):
            return True

        return self.to_dict() != other.to_dict()
