# AUTOGENERATED - modify shared_anyscale_util in root directory to make changes
from enum import Enum


class Bytes(int, Enum):
    """Gets the number of bytes per unit."""

    KB = 1024
    MB = 1024 * KB
    GB = 1024 * MB
