# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class DecoratedJobConnectionInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'workload_id': 'str',
        'workload_type': 'str',
        'connection': 'ConnectionInfo',
        'created_at': 'datetime',
        'created_by': 'MiniUser',
        'deleted_at': 'datetime',
        'deleted_by': 'MiniUser'
    }

    attribute_map = {
        'id': 'id',
        'workload_id': 'workload_id',
        'workload_type': 'workload_type',
        'connection': 'connection',
        'created_at': 'created_at',
        'created_by': 'created_by',
        'deleted_at': 'deleted_at',
        'deleted_by': 'deleted_by'
    }

    def __init__(self, id=None, workload_id=None, workload_type=None, connection=None, created_at=None, created_by=None, deleted_at=None, deleted_by=None, local_vars_configuration=None):  # noqa: E501
        """DecoratedJobConnectionInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._workload_id = None
        self._workload_type = None
        self._connection = None
        self._created_at = None
        self._created_by = None
        self._deleted_at = None
        self._deleted_by = None
        self.discriminator = None

        self.id = id
        self.workload_id = workload_id
        self.workload_type = workload_type
        self.connection = connection
        self.created_at = created_at
        self.created_by = created_by
        if deleted_at is not None:
            self.deleted_at = deleted_at
        if deleted_by is not None:
            self.deleted_by = deleted_by

    @property
    def id(self):
        """Gets the id of this DecoratedJobConnectionInfo.  # noqa: E501

        Workload connection ID  # noqa: E501

        :return: The id of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this DecoratedJobConnectionInfo.

        Workload connection ID  # noqa: E501

        :param id: The id of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def workload_id(self):
        """Gets the workload_id of this DecoratedJobConnectionInfo.  # noqa: E501

        Workload (job, workspace, etc) ID  # noqa: E501

        :return: The workload_id of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_id

    @workload_id.setter
    def workload_id(self, workload_id):
        """Sets the workload_id of this DecoratedJobConnectionInfo.

        Workload (job, workspace, etc) ID  # noqa: E501

        :param workload_id: The workload_id of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and workload_id is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_id`, must not be `None`")  # noqa: E501

        self._workload_id = workload_id

    @property
    def workload_type(self):
        """Gets the workload_type of this DecoratedJobConnectionInfo.  # noqa: E501

        Type of workload (job, workspace, etc.)  # noqa: E501

        :return: The workload_type of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: str
        """
        return self._workload_type

    @workload_type.setter
    def workload_type(self, workload_type):
        """Sets the workload_type of this DecoratedJobConnectionInfo.

        Type of workload (job, workspace, etc.)  # noqa: E501

        :param workload_type: The workload_type of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and workload_type is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_type`, must not be `None`")  # noqa: E501

        self._workload_type = workload_type

    @property
    def connection(self):
        """Gets the connection of this DecoratedJobConnectionInfo.  # noqa: E501

        Connection details  # noqa: E501

        :return: The connection of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: ConnectionInfo
        """
        return self._connection

    @connection.setter
    def connection(self, connection):
        """Sets the connection of this DecoratedJobConnectionInfo.

        Connection details  # noqa: E501

        :param connection: The connection of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: ConnectionInfo
        """
        if self.local_vars_configuration.client_side_validation and connection is None:  # noqa: E501
            raise ValueError("Invalid value for `connection`, must not be `None`")  # noqa: E501

        self._connection = connection

    @property
    def created_at(self):
        """Gets the created_at of this DecoratedJobConnectionInfo.  # noqa: E501

        When the connection was associated  # noqa: E501

        :return: The created_at of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this DecoratedJobConnectionInfo.

        When the connection was associated  # noqa: E501

        :param created_at: The created_at of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_at is None:  # noqa: E501
            raise ValueError("Invalid value for `created_at`, must not be `None`")  # noqa: E501

        self._created_at = created_at

    @property
    def created_by(self):
        """Gets the created_by of this DecoratedJobConnectionInfo.  # noqa: E501

        User who associated the connection  # noqa: E501

        :return: The created_by of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: MiniUser
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this DecoratedJobConnectionInfo.

        User who associated the connection  # noqa: E501

        :param created_by: The created_by of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: MiniUser
        """
        if self.local_vars_configuration.client_side_validation and created_by is None:  # noqa: E501
            raise ValueError("Invalid value for `created_by`, must not be `None`")  # noqa: E501

        self._created_by = created_by

    @property
    def deleted_at(self):
        """Gets the deleted_at of this DecoratedJobConnectionInfo.  # noqa: E501

        When the connection was disassociated  # noqa: E501

        :return: The deleted_at of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._deleted_at

    @deleted_at.setter
    def deleted_at(self, deleted_at):
        """Sets the deleted_at of this DecoratedJobConnectionInfo.

        When the connection was disassociated  # noqa: E501

        :param deleted_at: The deleted_at of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: datetime
        """

        self._deleted_at = deleted_at

    @property
    def deleted_by(self):
        """Gets the deleted_by of this DecoratedJobConnectionInfo.  # noqa: E501

        User who disassociated the connection  # noqa: E501

        :return: The deleted_by of this DecoratedJobConnectionInfo.  # noqa: E501
        :rtype: MiniUser
        """
        return self._deleted_by

    @deleted_by.setter
    def deleted_by(self, deleted_by):
        """Sets the deleted_by of this DecoratedJobConnectionInfo.

        User who disassociated the connection  # noqa: E501

        :param deleted_by: The deleted_by of this DecoratedJobConnectionInfo.  # noqa: E501
        :type: MiniUser
        """

        self._deleted_by = deleted_by

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DecoratedJobConnectionInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DecoratedJobConnectionInfo):
            return True

        return self.to_dict() != other.to_dict()
