# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class SSOConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'sso_connection': 'SSOConnection',
        'directory_sync': 'DirectorySync',
        'domain_verifications': 'list[DomainVerification]'
    }

    attribute_map = {
        'sso_connection': 'sso_connection',
        'directory_sync': 'directory_sync',
        'domain_verifications': 'domain_verifications'
    }

    def __init__(self, sso_connection=None, directory_sync=None, domain_verifications=None, local_vars_configuration=None):  # noqa: E501
        """SSOConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._sso_connection = None
        self._directory_sync = None
        self._domain_verifications = None
        self.discriminator = None

        self.sso_connection = sso_connection
        self.directory_sync = directory_sync
        self.domain_verifications = domain_verifications

    @property
    def sso_connection(self):
        """Gets the sso_connection of this SSOConfig.  # noqa: E501


        :return: The sso_connection of this SSOConfig.  # noqa: E501
        :rtype: SSOConnection
        """
        return self._sso_connection

    @sso_connection.setter
    def sso_connection(self, sso_connection):
        """Sets the sso_connection of this SSOConfig.


        :param sso_connection: The sso_connection of this SSOConfig.  # noqa: E501
        :type: SSOConnection
        """
        if self.local_vars_configuration.client_side_validation and sso_connection is None:  # noqa: E501
            raise ValueError("Invalid value for `sso_connection`, must not be `None`")  # noqa: E501

        self._sso_connection = sso_connection

    @property
    def directory_sync(self):
        """Gets the directory_sync of this SSOConfig.  # noqa: E501


        :return: The directory_sync of this SSOConfig.  # noqa: E501
        :rtype: DirectorySync
        """
        return self._directory_sync

    @directory_sync.setter
    def directory_sync(self, directory_sync):
        """Sets the directory_sync of this SSOConfig.


        :param directory_sync: The directory_sync of this SSOConfig.  # noqa: E501
        :type: DirectorySync
        """
        if self.local_vars_configuration.client_side_validation and directory_sync is None:  # noqa: E501
            raise ValueError("Invalid value for `directory_sync`, must not be `None`")  # noqa: E501

        self._directory_sync = directory_sync

    @property
    def domain_verifications(self):
        """Gets the domain_verifications of this SSOConfig.  # noqa: E501


        :return: The domain_verifications of this SSOConfig.  # noqa: E501
        :rtype: list[DomainVerification]
        """
        return self._domain_verifications

    @domain_verifications.setter
    def domain_verifications(self, domain_verifications):
        """Sets the domain_verifications of this SSOConfig.


        :param domain_verifications: The domain_verifications of this SSOConfig.  # noqa: E501
        :type: list[DomainVerification]
        """
        if self.local_vars_configuration.client_side_validation and domain_verifications is None:  # noqa: E501
            raise ValueError("Invalid value for `domain_verifications`, must not be `None`")  # noqa: E501

        self._domain_verifications = domain_verifications

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SSOConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SSOConfig):
            return True

        return self.to_dict() != other.to_dict()
