# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ListResourceQuotasQuery(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'TextQuery',
        'cloud_id': 'str',
        'creator_id': 'str',
        'is_enabled': 'bool',
        'paging': 'PageQuery'
    }

    attribute_map = {
        'name': 'name',
        'cloud_id': 'cloud_id',
        'creator_id': 'creator_id',
        'is_enabled': 'is_enabled',
        'paging': 'paging'
    }

    def __init__(self, name=None, cloud_id=None, creator_id=None, is_enabled=None, paging=None, local_vars_configuration=None):  # noqa: E501
        """ListResourceQuotasQuery - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._cloud_id = None
        self._creator_id = None
        self._is_enabled = None
        self._paging = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if cloud_id is not None:
            self.cloud_id = cloud_id
        if creator_id is not None:
            self.creator_id = creator_id
        if is_enabled is not None:
            self.is_enabled = is_enabled
        if paging is not None:
            self.paging = paging

    @property
    def name(self):
        """Gets the name of this ListResourceQuotasQuery.  # noqa: E501

        The name filter for the resource quotas.  # noqa: E501

        :return: The name of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: TextQuery
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this ListResourceQuotasQuery.

        The name filter for the resource quotas.  # noqa: E501

        :param name: The name of this ListResourceQuotasQuery.  # noqa: E501
        :type: TextQuery
        """

        self._name = name

    @property
    def cloud_id(self):
        """Gets the cloud_id of this ListResourceQuotasQuery.  # noqa: E501

        The cloud ID filter for the resource quotas.  # noqa: E501

        :return: The cloud_id of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this ListResourceQuotasQuery.

        The cloud ID filter for the resource quotas.  # noqa: E501

        :param cloud_id: The cloud_id of this ListResourceQuotasQuery.  # noqa: E501
        :type: str
        """

        self._cloud_id = cloud_id

    @property
    def creator_id(self):
        """Gets the creator_id of this ListResourceQuotasQuery.  # noqa: E501

        The creator ID filter for the resource quotas.  # noqa: E501

        :return: The creator_id of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: str
        """
        return self._creator_id

    @creator_id.setter
    def creator_id(self, creator_id):
        """Sets the creator_id of this ListResourceQuotasQuery.

        The creator ID filter for the resource quotas.  # noqa: E501

        :param creator_id: The creator_id of this ListResourceQuotasQuery.  # noqa: E501
        :type: str
        """

        self._creator_id = creator_id

    @property
    def is_enabled(self):
        """Gets the is_enabled of this ListResourceQuotasQuery.  # noqa: E501

        The is_enabled filter for the resource quotas.  # noqa: E501

        :return: The is_enabled of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: bool
        """
        return self._is_enabled

    @is_enabled.setter
    def is_enabled(self, is_enabled):
        """Sets the is_enabled of this ListResourceQuotasQuery.

        The is_enabled filter for the resource quotas.  # noqa: E501

        :param is_enabled: The is_enabled of this ListResourceQuotasQuery.  # noqa: E501
        :type: bool
        """

        self._is_enabled = is_enabled

    @property
    def paging(self):
        """Gets the paging of this ListResourceQuotasQuery.  # noqa: E501

        Pagination information.  # noqa: E501

        :return: The paging of this ListResourceQuotasQuery.  # noqa: E501
        :rtype: PageQuery
        """
        return self._paging

    @paging.setter
    def paging(self, paging):
        """Sets the paging of this ListResourceQuotasQuery.

        Pagination information.  # noqa: E501

        :param paging: The paging of this ListResourceQuotasQuery.  # noqa: E501
        :type: PageQuery
        """

        self._paging = paging

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ListResourceQuotasQuery):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ListResourceQuotasQuery):
            return True

        return self.to_dict() != other.to_dict()
