# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationContractInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'organization_name': 'str',
        'metronome_customer_id': 'str',
        'customer_type': 'str',
        'contracts': 'list[ContractInfo]',
        'current_contract_type': 'str',
        'poc_end_date': 'datetime'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'organization_name': 'organization_name',
        'metronome_customer_id': 'metronome_customer_id',
        'customer_type': 'customer_type',
        'contracts': 'contracts',
        'current_contract_type': 'current_contract_type',
        'poc_end_date': 'poc_end_date'
    }

    def __init__(self, organization_id=None, organization_name=None, metronome_customer_id=None, customer_type=None, contracts=None, current_contract_type=None, poc_end_date=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationContractInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._organization_name = None
        self._metronome_customer_id = None
        self._customer_type = None
        self._contracts = None
        self._current_contract_type = None
        self._poc_end_date = None
        self.discriminator = None

        self.organization_id = organization_id
        self.organization_name = organization_name
        self.metronome_customer_id = metronome_customer_id
        if customer_type is not None:
            self.customer_type = customer_type
        if contracts is not None:
            self.contracts = contracts
        if current_contract_type is not None:
            self.current_contract_type = current_contract_type
        if poc_end_date is not None:
            self.poc_end_date = poc_end_date

    @property
    def organization_id(self):
        """Gets the organization_id of this OrganizationContractInfo.  # noqa: E501

        ID of the organization  # noqa: E501

        :return: The organization_id of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this OrganizationContractInfo.

        ID of the organization  # noqa: E501

        :param organization_id: The organization_id of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def organization_name(self):
        """Gets the organization_name of this OrganizationContractInfo.  # noqa: E501

        Name of the organization  # noqa: E501

        :return: The organization_name of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._organization_name

    @organization_name.setter
    def organization_name(self, organization_name):
        """Sets the organization_name of this OrganizationContractInfo.

        Name of the organization  # noqa: E501

        :param organization_name: The organization_name of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_name is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_name`, must not be `None`")  # noqa: E501

        self._organization_name = organization_name

    @property
    def metronome_customer_id(self):
        """Gets the metronome_customer_id of this OrganizationContractInfo.  # noqa: E501

        Metronome customer ID  # noqa: E501

        :return: The metronome_customer_id of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._metronome_customer_id

    @metronome_customer_id.setter
    def metronome_customer_id(self, metronome_customer_id):
        """Sets the metronome_customer_id of this OrganizationContractInfo.

        Metronome customer ID  # noqa: E501

        :param metronome_customer_id: The metronome_customer_id of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and metronome_customer_id is None:  # noqa: E501
            raise ValueError("Invalid value for `metronome_customer_id`, must not be `None`")  # noqa: E501

        self._metronome_customer_id = metronome_customer_id

    @property
    def customer_type(self):
        """Gets the customer_type of this OrganizationContractInfo.  # noqa: E501

        Customer type  # noqa: E501

        :return: The customer_type of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._customer_type

    @customer_type.setter
    def customer_type(self, customer_type):
        """Sets the customer_type of this OrganizationContractInfo.

        Customer type  # noqa: E501

        :param customer_type: The customer_type of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """

        self._customer_type = customer_type

    @property
    def contracts(self):
        """Gets the contracts of this OrganizationContractInfo.  # noqa: E501

        List of contracts  # noqa: E501

        :return: The contracts of this OrganizationContractInfo.  # noqa: E501
        :rtype: list[ContractInfo]
        """
        return self._contracts

    @contracts.setter
    def contracts(self, contracts):
        """Sets the contracts of this OrganizationContractInfo.

        List of contracts  # noqa: E501

        :param contracts: The contracts of this OrganizationContractInfo.  # noqa: E501
        :type: list[ContractInfo]
        """

        self._contracts = contracts

    @property
    def current_contract_type(self):
        """Gets the current_contract_type of this OrganizationContractInfo.  # noqa: E501

        Type of current active contract  # noqa: E501

        :return: The current_contract_type of this OrganizationContractInfo.  # noqa: E501
        :rtype: str
        """
        return self._current_contract_type

    @current_contract_type.setter
    def current_contract_type(self, current_contract_type):
        """Sets the current_contract_type of this OrganizationContractInfo.

        Type of current active contract  # noqa: E501

        :param current_contract_type: The current_contract_type of this OrganizationContractInfo.  # noqa: E501
        :type: str
        """

        self._current_contract_type = current_contract_type

    @property
    def poc_end_date(self):
        """Gets the poc_end_date of this OrganizationContractInfo.  # noqa: E501

        POC end date if in POC  # noqa: E501

        :return: The poc_end_date of this OrganizationContractInfo.  # noqa: E501
        :rtype: datetime
        """
        return self._poc_end_date

    @poc_end_date.setter
    def poc_end_date(self, poc_end_date):
        """Sets the poc_end_date of this OrganizationContractInfo.

        POC end date if in POC  # noqa: E501

        :param poc_end_date: The poc_end_date of this OrganizationContractInfo.  # noqa: E501
        :type: datetime
        """

        self._poc_end_date = poc_end_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationContractInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationContractInfo):
            return True

        return self.to_dict() != other.to_dict()
