# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class KubernetesConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'anyscale_operator_iam_identity': 'str',
        'zones': 'list[str]'
    }

    attribute_map = {
        'anyscale_operator_iam_identity': 'anyscale_operator_iam_identity',
        'zones': 'zones'
    }

    def __init__(self, anyscale_operator_iam_identity=None, zones=None, local_vars_configuration=None):  # noqa: E501
        """KubernetesConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._anyscale_operator_iam_identity = None
        self._zones = None
        self.discriminator = None

        if anyscale_operator_iam_identity is not None:
            self.anyscale_operator_iam_identity = anyscale_operator_iam_identity
        if zones is not None:
            self.zones = zones

    @property
    def anyscale_operator_iam_identity(self):
        """Gets the anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501

        The cloud provider IAM identity federated with the Anyscale Operator's Kubernetes service account, which will be used by Anyscale control plane for validation during Anyscale Operator bootstrap in the dataplane. IN AWS EKS, this is the ARN of the IAM role. For GCP GKE, this is the service account email.  # noqa: E501

        :return: The anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501
        :rtype: str
        """
        return self._anyscale_operator_iam_identity

    @anyscale_operator_iam_identity.setter
    def anyscale_operator_iam_identity(self, anyscale_operator_iam_identity):
        """Sets the anyscale_operator_iam_identity of this KubernetesConfig.

        The cloud provider IAM identity federated with the Anyscale Operator's Kubernetes service account, which will be used by Anyscale control plane for validation during Anyscale Operator bootstrap in the dataplane. IN AWS EKS, this is the ARN of the IAM role. For GCP GKE, this is the service account email.  # noqa: E501

        :param anyscale_operator_iam_identity: The anyscale_operator_iam_identity of this KubernetesConfig.  # noqa: E501
        :type: str
        """

        self._anyscale_operator_iam_identity = anyscale_operator_iam_identity

    @property
    def zones(self):
        """Gets the zones of this KubernetesConfig.  # noqa: E501

        List of zones to launch pods in.  # noqa: E501

        :return: The zones of this KubernetesConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._zones

    @zones.setter
    def zones(self, zones):
        """Sets the zones of this KubernetesConfig.

        List of zones to launch pods in.  # noqa: E501

        :param zones: The zones of this KubernetesConfig.  # noqa: E501
        :type: list[str]
        """

        self._zones = zones

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, KubernetesConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, KubernetesConfig):
            return True

        return self.to_dict() != other.to_dict()
