# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class ConvertToSelfServiceRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'organization_id': 'str',
        'self_service_start_date': 'datetime',
        'end_current_contract': 'bool'
    }

    attribute_map = {
        'organization_id': 'organization_id',
        'self_service_start_date': 'self_service_start_date',
        'end_current_contract': 'end_current_contract'
    }

    def __init__(self, organization_id=None, self_service_start_date=None, end_current_contract=True, local_vars_configuration=None):  # noqa: E501
        """ConvertToSelfServiceRequest - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._organization_id = None
        self._self_service_start_date = None
        self._end_current_contract = None
        self.discriminator = None

        self.organization_id = organization_id
        if self_service_start_date is not None:
            self.self_service_start_date = self_service_start_date
        if end_current_contract is not None:
            self.end_current_contract = end_current_contract

    @property
    def organization_id(self):
        """Gets the organization_id of this ConvertToSelfServiceRequest.  # noqa: E501

        ID of the organization  # noqa: E501

        :return: The organization_id of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this ConvertToSelfServiceRequest.

        ID of the organization  # noqa: E501

        :param organization_id: The organization_id of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and organization_id is None:  # noqa: E501
            raise ValueError("Invalid value for `organization_id`, must not be `None`")  # noqa: E501

        self._organization_id = organization_id

    @property
    def self_service_start_date(self):
        """Gets the self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501

        When self-service should start (defaults to now)  # noqa: E501

        :return: The self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: datetime
        """
        return self._self_service_start_date

    @self_service_start_date.setter
    def self_service_start_date(self, self_service_start_date):
        """Sets the self_service_start_date of this ConvertToSelfServiceRequest.

        When self-service should start (defaults to now)  # noqa: E501

        :param self_service_start_date: The self_service_start_date of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: datetime
        """

        self._self_service_start_date = self_service_start_date

    @property
    def end_current_contract(self):
        """Gets the end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501

        Whether to end current contract  # noqa: E501

        :return: The end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501
        :rtype: bool
        """
        return self._end_current_contract

    @end_current_contract.setter
    def end_current_contract(self, end_current_contract):
        """Sets the end_current_contract of this ConvertToSelfServiceRequest.

        Whether to end current contract  # noqa: E501

        :param end_current_contract: The end_current_contract of this ConvertToSelfServiceRequest.  # noqa: E501
        :type: bool
        """

        self._end_current_contract = end_current_contract

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConvertToSelfServiceRequest):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ConvertToSelfServiceRequest):
            return True

        return self.to_dict() != other.to_dict()
