# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class RequestStateInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'size': 'int',
        'machine_type': 'str',
        'workload_info': 'WorkloadInfo',
        'partition_scores': 'dict(str, int)'
    }

    attribute_map = {
        'size': 'size',
        'machine_type': 'machine_type',
        'workload_info': 'workload_info',
        'partition_scores': 'partition_scores'
    }

    def __init__(self, size=None, machine_type=None, workload_info=None, partition_scores=None, local_vars_configuration=None):  # noqa: E501
        """RequestStateInfo - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._size = None
        self._machine_type = None
        self._workload_info = None
        self._partition_scores = None
        self.discriminator = None

        self.size = size
        self.machine_type = machine_type
        self.workload_info = workload_info
        self.partition_scores = partition_scores

    @property
    def size(self):
        """Gets the size of this RequestStateInfo.  # noqa: E501

        The size of the request.  # noqa: E501

        :return: The size of this RequestStateInfo.  # noqa: E501
        :rtype: int
        """
        return self._size

    @size.setter
    def size(self, size):
        """Sets the size of this RequestStateInfo.

        The size of the request.  # noqa: E501

        :param size: The size of this RequestStateInfo.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and size is None:  # noqa: E501
            raise ValueError("Invalid value for `size`, must not be `None`")  # noqa: E501

        self._size = size

    @property
    def machine_type(self):
        """Gets the machine_type of this RequestStateInfo.  # noqa: E501

        The type of the machine.  # noqa: E501

        :return: The machine_type of this RequestStateInfo.  # noqa: E501
        :rtype: str
        """
        return self._machine_type

    @machine_type.setter
    def machine_type(self, machine_type):
        """Sets the machine_type of this RequestStateInfo.

        The type of the machine.  # noqa: E501

        :param machine_type: The machine_type of this RequestStateInfo.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and machine_type is None:  # noqa: E501
            raise ValueError("Invalid value for `machine_type`, must not be `None`")  # noqa: E501

        self._machine_type = machine_type

    @property
    def workload_info(self):
        """Gets the workload_info of this RequestStateInfo.  # noqa: E501

        Workload metadata associated with this request.  # noqa: E501

        :return: The workload_info of this RequestStateInfo.  # noqa: E501
        :rtype: WorkloadInfo
        """
        return self._workload_info

    @workload_info.setter
    def workload_info(self, workload_info):
        """Sets the workload_info of this RequestStateInfo.

        Workload metadata associated with this request.  # noqa: E501

        :param workload_info: The workload_info of this RequestStateInfo.  # noqa: E501
        :type: WorkloadInfo
        """
        if self.local_vars_configuration.client_side_validation and workload_info is None:  # noqa: E501
            raise ValueError("Invalid value for `workload_info`, must not be `None`")  # noqa: E501

        self._workload_info = workload_info

    @property
    def partition_scores(self):
        """Gets the partition_scores of this RequestStateInfo.  # noqa: E501

        The scores of the partitions.  # noqa: E501

        :return: The partition_scores of this RequestStateInfo.  # noqa: E501
        :rtype: dict(str, int)
        """
        return self._partition_scores

    @partition_scores.setter
    def partition_scores(self, partition_scores):
        """Sets the partition_scores of this RequestStateInfo.

        The scores of the partitions.  # noqa: E501

        :param partition_scores: The partition_scores of this RequestStateInfo.  # noqa: E501
        :type: dict(str, int)
        """
        if self.local_vars_configuration.client_side_validation and partition_scores is None:  # noqa: E501
            raise ValueError("Invalid value for `partition_scores`, must not be `None`")  # noqa: E501

        self._partition_scores = partition_scores

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RequestStateInfo):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RequestStateInfo):
            return True

        return self.to_dict() != other.to_dict()
