# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class CreateSSOConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'idp_metadata_url': 'str',
        'static_idp_config': 'StaticSSOConfig'
    }

    attribute_map = {
        'idp_metadata_url': 'idp_metadata_url',
        'static_idp_config': 'static_idp_config'
    }

    def __init__(self, idp_metadata_url=None, static_idp_config=None, local_vars_configuration=None):  # noqa: E501
        """CreateSSOConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._idp_metadata_url = None
        self._static_idp_config = None
        self.discriminator = None

        if idp_metadata_url is not None:
            self.idp_metadata_url = idp_metadata_url
        if static_idp_config is not None:
            self.static_idp_config = static_idp_config

    @property
    def idp_metadata_url(self):
        """Gets the idp_metadata_url of this CreateSSOConfig.  # noqa: E501

        Identity provider (IdP) metadata url. If given along with the other static identity provider fields, Anyscale will first attempt metadata exchange to get IdP attributes. If Anyscale can't reach the metadata endpoint or if the metadata url isn't given, Anyscale will use static_idp_config as a backup. Either idp_metadata_url or static_idp_config is required.  # noqa: E501

        :return: The idp_metadata_url of this CreateSSOConfig.  # noqa: E501
        :rtype: str
        """
        return self._idp_metadata_url

    @idp_metadata_url.setter
    def idp_metadata_url(self, idp_metadata_url):
        """Sets the idp_metadata_url of this CreateSSOConfig.

        Identity provider (IdP) metadata url. If given along with the other static identity provider fields, Anyscale will first attempt metadata exchange to get IdP attributes. If Anyscale can't reach the metadata endpoint or if the metadata url isn't given, Anyscale will use static_idp_config as a backup. Either idp_metadata_url or static_idp_config is required.  # noqa: E501

        :param idp_metadata_url: The idp_metadata_url of this CreateSSOConfig.  # noqa: E501
        :type: str
        """

        self._idp_metadata_url = idp_metadata_url

    @property
    def static_idp_config(self):
        """Gets the static_idp_config of this CreateSSOConfig.  # noqa: E501

        Static identity provider configuration.  # noqa: E501

        :return: The static_idp_config of this CreateSSOConfig.  # noqa: E501
        :rtype: StaticSSOConfig
        """
        return self._static_idp_config

    @static_idp_config.setter
    def static_idp_config(self, static_idp_config):
        """Sets the static_idp_config of this CreateSSOConfig.

        Static identity provider configuration.  # noqa: E501

        :param static_idp_config: The static_idp_config of this CreateSSOConfig.  # noqa: E501
        :type: StaticSSOConfig
        """

        self._static_idp_config = static_idp_config

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateSSOConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateSSOConfig):
            return True

        return self.to_dict() != other.to_dict()
